@@ -5,7 +5,7 @@ Implements the JVC protocol
 import logging
 from typing import Final, Union
 import asyncio
-from jvc_projector.commands import InputLevel, ColorSpaceModes,EshiftModes ,ACKs, Footer, Header, Commands, PowerStates, PictureModes, InstallationModes, InputModes, LaserDimModes, Enum, LowLatencyModes
+from jvc_projector.commands import ACKs, Footer, Header, Commands, PowerStates, Enum
 
 
 class JVCProjector:
@@ -27,6 +27,11 @@ class JVCProjector:
         self.password = password
         self.connect_timeout: int = connect_timeout
         self.logger = logger
+        # use the provided loop or get current one. Otherwise make one
+        try:
+            self._loop = loop or asyncio.get_running_loop()
+        except RuntimeError:
+            self._loop = asyncio.new_event_loop()
         self._lock = asyncio.Lock()
         # Const values
         self.PJ_OK: Final = ACKs.greeting.value
@@ -47,35 +52,35 @@ class JVCProjector:
         return success
 
     async def reconnect(self):
-        """Initiate keep-alive connection. This should handle any error and reconnect eventually."""
+        """Initiate keep-alive connection"""
         while True:
             try:
-                if self.writer is not None:
-                    self.logger.debug("Closing writer")
-                    self.writer.close()
-                    await self.writer.wait_closed()
-                self.logger.info(
+                self.logger.debug(
                     "Connecting to JVC Projector: %s:%s", self.host, self.port
                 )
-                cor = asyncio.open_connection(self.host, self.port)
-                # wait for 10 sec to connect
-                self.reader, self.writer = await asyncio.wait_for(cor, 10)
-                self.logger.info("Connected to JVC Projector")
-                # create a reader and writer to do handshake
-                self.logger.debug("Handshaking")
-                result, success = await self._async_handshake()
-                if not success:
-                    return result, success
-                self.logger.debug("Handshake complete and we are connected")
+                async with self._lock:
+                    # transport, protocol = await self._loop.create_connection(
+                    #     lambda: self.protocol, self.host, self.port
+                    # )
+                    cor = asyncio.open_connection(self.host, self.port, loop=self._loop)
+                    # wait for 10 sec to connect
+                    self.reader, self.writer = await asyncio.wait_for(cor, 10)
+                    self.logger.debug("Connected to JVC Projector")
+                    # create a reader and writer to do handshake
+                async with self._lock:
+                    self.logger.debug("Handshaking")
+                    result, success = await self._async_handshake()
+                    if not success:
+                        return result, success
+                    self.logger.debug("Handshake complete and we are connected")
                 return "Connection done", True
 
             # includes conn refused
-            except OSError as err:
-                self.logger.warning("Connecting failed, retrying in 2 seconds")
-                self.logger.debug(err)
+            except OSError:
+                self.logger.warning("Connecting failed, retrying in %i seconds", 2)
                 await asyncio.sleep(2)
             except asyncio.TimeoutError:
-                self.logger.warning("Connection timed out, retrying in 2 seconds")
+                self.logger.warning("Connection timed out, retrying in %i seconds", 2)
                 await asyncio.sleep(2)
 
     async def _async_handshake(self) -> tuple[str, bool]:
@@ -194,94 +199,95 @@ class JVCProjector:
     ) -> tuple[str, bool]:
         retry_count = 0
         while retry_count < 5:
-            self.logger.debug("do_command sending command: %s", command)
-            # send the command
-            self.writer.write(command)
-            try:
-                await self.writer.drain()
-            except ConnectionError as err:
-                # reaching this means the writer was closed somewhere
-                self.logger.error(err)
-                self.logger.debug("Restarting connection")
-                # restart the connection
+            async with self._lock:
+                self.logger.debug("do_command sending command: %s", command)
+                # send the command
+                self.writer.write(command)
+                try:
+                    await self.writer.drain()
+                except ConnectionError as err:
+                    # reaching this means the writer was closed somewhere
+                    self.logger.error(err)
+                    self.logger.debug("Restarting connection")
+                    # restart the connection
 
-                await self.reconnect()
-                self.logger.debug("Sending command again")
-                # restart the loop
-                retry_count += 1
-                continue
+                    await self.reconnect()
+                    self.logger.debug("Sending command again")
+                    # restart the loop
+                    retry_count += 1
+                    continue
 
-            # if we send a command that returns info, the projector will send
-            # an ack, followed by the actual message. Check to see if the ack sent by
-            # projector is correct, then return the message.
-            ack_value = (
-                Header.ack.value + Header.pj_unit.value + ack + Footer.close.value
-            )
-            self.logger.debug("constructed ack_value: %s", ack_value)
-
-            # Receive the acknowledgement from PJ
-            try:
-                # seems like certain commands timeout when PJ is off
-                received_ack = await asyncio.wait_for(
-                    self.reader.readline(), timeout=self.command_read_timeout
+                # if we send a command that returns info, the projector will send
+                # an ack, followed by the actual message. Check to see if the ack sent by
+                # projector is correct, then return the message.
+                ack_value = (
+                    Header.ack.value + Header.pj_unit.value + ack + Footer.close.value
                 )
-            except asyncio.TimeoutError:
-                # LL is used in async_update() and I don't want to spam HA logs so we skip
-                # if not command == b"?\x89\x01PMLL\n":
-                # Sometimes if you send a command that is greyed out, the PJ will just hang
+                self.logger.debug("constructed ack_value: %s", ack_value)
+
+                # Receive the acknowledgement from PJ
+                try:
+                    # seems like certain commands timeout when PJ is off
+                    received_ack = await asyncio.wait_for(
+                        self.reader.readline(), timeout=self.command_read_timeout
+                    )
+                except asyncio.TimeoutError:
+                    # LL is used in async_update() and I don't want to spam HA logs so we skip
+                    # if not command == b"?\x89\x01PMLL\n":
+                    # Sometimes if you send a command that is greyed out, the PJ will just hang
+                    self.logger.error(
+                        "Connection timed out. Command %s is probably not allowed to run at this time.",
+                        command,
+                    )
+                    self.logger.debug("restarting connection")
+
+                    await self.reconnect()
+                    retry_count += 1
+                    continue
+
+                except ConnectionRefusedError:
+                    self.logger.error("Connection Refused when getting ack")
+                    self.logger.debug("restarting connection")
+
+                    await self.reconnect()
+                    retry_count += 1
+                    continue
+
+                self.logger.debug("received ack from PJ: %s", received_ack)
+
+                # This will probably never happen since we are handling timeouts now
+                if received_ack == b"":
+                    self.logger.error("Got a blank ack. Restarting connection")
+
+                    await self.reconnect()
+                    retry_count += 1
+                    continue
+
+                # get the ack for operation
+                if received_ack == ack_value and command_type == Header.operation.value:
+                    return received_ack, True
+
+                # if we got what we expect and this is a reference,
+                # receive the data we requested
+                if received_ack == ack_value and command_type == Header.reference.value:
+                    message = await self.reader.readline()
+                    self.logger.debug("received message from PJ: %s", message)
+
+                    return message, True
+
+                # Otherwise, it failed
+                # Because this now reuses a connection, reaching this stage means catastrophic failure, or HA running as usual :)
                 self.logger.error(
-                    "Connection timed out. Command %s is probably not allowed to run at this time.",
-                    command,
+                    "Recieved ack did not match expected ack: %s != %s",
+                    received_ack,
+                    ack_value,
                 )
-                self.logger.debug("restarting connection")
+                # Try to restart connection, if we got here somethihng is out of sync
 
                 await self.reconnect()
                 retry_count += 1
                 continue
 
-            except ConnectionRefusedError:
-                self.logger.error("Connection Refused when getting ack")
-                self.logger.debug("restarting connection")
-
-                await self.reconnect()
-                retry_count += 1
-                continue
-
-            self.logger.debug("received ack from PJ: %s", received_ack)
-
-            # This will probably never happen since we are handling timeouts now
-            if received_ack == b"":
-                self.logger.error("Got a blank ack. Restarting connection")
-
-                await self.reconnect()
-                retry_count += 1
-                continue
-
-            # get the ack for operation
-            if received_ack == ack_value and command_type == Header.operation.value:
-                return received_ack, True
-
-            # if we got what we expect and this is a reference,
-            # receive the data we requested
-            if received_ack == ack_value and command_type == Header.reference.value:
-                message = await self.reader.readline()
-                self.logger.debug("received message from PJ: %s", message)
-
-                return message, True
-
-            # Otherwise, it failed
-            # Because this now reuses a connection, reaching this stage means catastrophic failure, or HA running as usual :)
-            self.logger.error(
-                "Recieved ack did not match expected ack: %s != %s",
-                received_ack,
-                ack_value,
-            )
-            # Try to restart connection, if we got here somethihng is out of sync
-
-            await self.reconnect()
-            retry_count += 1
-            continue
-
         self.logger.error("retry count for running commands exceeded")
         return "retry count exceeded", False
 
@@ -388,16 +394,6 @@ class JVCProjector:
         Turns off PJ
         """
         return await self.async_exec_command("power,off")
-    
-    async def _async_replace_headers(self, item: bytes) -> bytes:
-        """
-        Will strip all headers and returns the value itself
-        """
-        headers = [x.value for x in Header] + [x.value for x in Footer]
-        for header in headers:
-            item = item.replace(header, b"")
-
-        return item
 
     async def _async_do_reference_op(self, command: str, ack: ACKs) -> tuple[str, bool]:
         cmd = (
@@ -426,75 +422,64 @@ class JVCProjector:
     async def async_get_low_latency_state(self) -> bool:
         """
         Get the current state of LL
+
+        None if there was an error
         """
-        state, _ = await self._async_do_reference_op(
+        state, success = await self._async_do_reference_op(
             "low_latency", ACKs.picture_ack
         )
+        # LL is off, could be disabled
+        if not success:
+            return None
+        if state == b"PM0":
+            return False
 
-        return LowLatencyModes(state.replace(ACKs.picture_ack.value, b"")).name
-    
-    async def async_get_picture_mode(self) -> str:
+        return True
+
+    # async def _async_check_low_latency(self) -> list[str]:
+    #     """
+    #     Infer if Low Latency is disabled or not otherwise commands will hang
+
+    #     Returns enabled modes that have to be disabled first
+    #     """
+    #     enabled_modes = []
+
+    #     state = await self.get_low_latency_state()
+    #     # LL is off, could be disabled
+    #     if state:
+    #         state, success = await self._async_do_reference_op(
+    #             "laser_dim", ACKs.picture_ack
+    #         )
+    #         if success:
+    #             if state != b"PM0":
+    #                 self.logger.debug("Laser dimming is enabled")
+    #                 enabled_modes.append("laser_dim")
+    #         else:
+    #             return "Error", state
+    #         # see if its hdr10+ or frame adapt
+    #         state, success = await self._async_do_reference_op(
+    #             "picture_mode", ACKs.picture_ack
+    #         )
+    #         if success:
+    #             if state in [b"PM0B", b"PM15"]:
+    #                 self.logger.debug("Locked HDR is enabled")
+    #                 enabled_modes.append("picture_mode")
+    #         else:
+    #             return "Error", state
+
+    #     self.logger.debug("Low Latency check: %s", success)
+
+    #     return enabled_modes
+
+    async def _async_replace_headers(self, item: bytes) -> bytes:
         """
-        Get the current picture mode as str -> user1, natural
+        Will strip all headers and returns the value itself
         """
-        state, _ = await self._async_do_reference_op(
-            "picture_mode", ACKs.picture_ack
-        )
-        return PictureModes(state.replace(ACKs.picture_ack.value, b"")).name
-    
-    async def async_get_install_mode(self) -> str:
-        """
-        Get the current install mode as str
-        """
-        state, _ = await self._async_do_reference_op(
-            "installation_mode", ACKs.install_acks
-        )
-        return InstallationModes(state.replace(ACKs.install_acks.value, b"")).name
-    
-    async def async_get_input_mode(self) -> str:
-        """
-        Get the current input mode
-        """
-        state, _ = await self._async_do_reference_op(
-            "input_mode", ACKs.input_ack
-        )
-        return InputModes(state.replace(ACKs.input_ack.value, b"")).name
-    
-    async def async_get_laser_mode(self) -> str:
-        """
-        Get the current laser mode
-        """
-        state, _ = await self._async_do_reference_op(
-            "laser_mode", ACKs.picture_ack
-        )
-        return LaserDimModes(state.replace(ACKs.picture_ack.value, b"")).name
-    
-    async def async_get_eshift_mode(self) -> str:
-        """
-        Get the current eshift mode
-        """
-        state, _ = await self._async_do_reference_op(
-            "eshift_mode", ACKs.picture_ack
-        )
-        return EshiftModes(state.replace(ACKs.picture_ack.value, b"")).name
-   
-    async def async_get_color_mode(self) -> str:
-        """
-        Get the current color mode
-        """
-        state, _ = await self._async_do_reference_op(
-            "color_mode", ACKs.hdmi_ack
-        )
-        return ColorSpaceModes(state.replace(ACKs.hdmi_ack.value, b"")).name
-    
-    async def async_get_input_level(self) -> str:
-        """
-        Get the current input level
-        """
-        state, _ = await self._async_do_reference_op(
-            "input_level", ACKs.hdmi_ack
-        )
-        return InputLevel(state.replace(ACKs.hdmi_ack.value, b"")).name
+        headers = [x.value for x in Header] + [x.value for x in Footer]
+        for header in headers:
+            item = item.replace(header, b"")
+
+        return item
 
     async def _async_get_power_state(self) -> str:
         """
@@ -532,14 +517,8 @@ class JVCProjector:
         """
         True if the current state is on|reserved
         """
-        pw_status = [PowerStates.on.name]
+        pw_status = [PowerStates.on.name, PowerStates.reserved.name]
         return await self._async_get_power_state() in pw_status
-    
-    async def async_is_ll_on(self) -> bool:
-        """
-        True if LL mode is on
-        """
-        return await self.async_get_low_latency_state() == LowLatencyModes.on.name
 
     def print_commands(self) -> str:
         """
